@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *
- *   Copyright 1996-2016 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2017 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -44,11 +44,12 @@
 
 #include "nasm.h"
 #include "nasmlib.h"
+#include "error.h"
 #include "preproc.h"
 #include "saa.h"
 #include "hashtbl.h"
-#include "output/outlib.h"
-#include "output/pecoff.h"
+#include "outlib.h"
+#include "pecoff.h"
 #include "md5.h"
 
 static void cv8_init(void);
@@ -60,7 +61,7 @@ static void cv8_typevalue(int32_t type);
 static void cv8_output(int type, void *param);
 static void cv8_cleanup(void);
 
-struct dfmt df_cv8 = {
+const struct dfmt df_cv8 = {
     "Codeview 8",               /* .fullname */
     "cv8",                      /* .shortname */
     cv8_init,                   /* .init */
@@ -70,6 +71,7 @@ struct dfmt df_cv8 = {
     cv8_typevalue,              /* .debug_typevalue */
     cv8_output,                 /* .debug_output */
     cv8_cleanup,                /* .cleanup */
+    NULL                        /* pragma list */
 };
 
 /*******************************************************************************
@@ -280,6 +282,9 @@ static void cv8_typevalue(int32_t type)
     case TY_YWORD:
         cv8_state.last_sym->symtype = TYPE_REAL256;
         break;
+    case TY_ZWORD:
+        cv8_state.last_sym->symtype = TYPE_REAL512;
+        break;
     case TY_UNKNOWN:
         break;
     case TY_LABEL:
@@ -341,7 +346,7 @@ static void calc_md5(const char *const filename,
     FILE *f;
     MD5_CTX ctx;
 
-    f = pp_input_fopen(filename, "rb");
+    f = pp_input_fopen(filename, NF_BINARY);
     if (!f)
         goto done;
 
@@ -436,6 +441,7 @@ static void register_reloc(struct coff_Section *const sect,
 {
     struct coff_Reloc *r;
     struct coff_Section *sec;
+    uint32_t i;
 
     r = *sect->tail = nasm_malloc(sizeof(struct coff_Reloc));
     sect->tail = &r->next;
@@ -447,7 +453,7 @@ static void register_reloc(struct coff_Section *const sect,
     r->type = type;
 
     r->symbol = 0;
-    for (int i = 0; i < coff_nsects; i++) {
+    for (i = 0; i < (uint32_t)coff_nsects; i++) {
         sec = coff_sects[i];
         if (!strcmp(sym, sec->name)) {
             return;
@@ -456,7 +462,7 @@ static void register_reloc(struct coff_Section *const sect,
     }
 
     saa_rewind(coff_syms);
-    for (uint32_t i = 0; i < coff_nsyms; i++) {
+    for (i = 0; i < coff_nsyms; i++) {
         struct coff_Symbol *s = saa_rstruct(coff_syms);
         r->symbol++;
         if (s->strpos == -1 && !strcmp(sym, s->name)) {
@@ -624,22 +630,30 @@ static uint16_t write_symbolinfo_obj(struct coff_Section *sect)
 static uint16_t write_symbolinfo_properties(struct coff_Section *sect,
         const char *const creator_str)
 {
+    /* https://github.com/Microsoft/microsoft-pdb/blob/1d60e041/include/cvinfo.h#L3313 */
     uint16_t creator_len;
 
-    creator_len = 2 + 4 + 4 + 4 + 4 + strlen(creator_str)+1 + 2;
+    creator_len = 2 + 4 + 2 + 3*2 + 3*2 + strlen(creator_str)+1 + 2;
 
     section_write16(sect, creator_len);
     section_write16(sect, 0x1116);
-    section_write32(sect, 3); /* language */
+    section_write32(sect, 3); /* language/flags */
     if (win64)
-        section_write32(sect, 0x000000D0);
+        section_write16(sect, 0x00D0); /* machine */
     else if (win32)
-        section_write32(sect, 0x00000006);
+        section_write16(sect, 0x0006); /* machine */
     else
         nasm_assert(!"neither win32 nor win64 are set!");
-    section_write32(sect, 0); /* flags*/
-    section_write32(sect, 8); /* version */
-    section_wbytes(sect, creator_str, strlen(creator_str)+1);
+    section_write16(sect, 0); /* verFEMajor */
+    section_write16(sect, 0); /* verFEMinor */
+    section_write16(sect, 0); /* verFEBuild */
+
+    /* BinScope/WACK insist on version >= 8.0.50727 */
+    section_write16(sect, 8); /* verMajor */
+    section_write16(sect, 0); /* verMinor */
+    section_write16(sect, 50727); /* verBuild */
+
+    section_wbytes(sect, creator_str, strlen(creator_str)+1); /* verSt */
     /*
      * normally there would be key/value pairs here, but they aren't
      * necessary. They are terminated by 2B
@@ -712,7 +726,7 @@ static void write_symbolinfo_table(struct coff_Section *const sect)
 
     /* signature, language, outfile NULL */
     obj_length = 2 + 4 + cv8_state.outfile.namebytes;
-    creator_length = 2 + 4 + 4 + 4 + 4 + strlen(creator_str)+1 + 2;
+    creator_length = 2 + 4 + 2 + 3*2 + 3*2 + strlen(creator_str)+1 + 2;
 
     sym_length =    ( cv8_state.num_syms[SYMTYPE_CODE] *  7) +
             ( cv8_state.num_syms[SYMTYPE_PROC] *  7) +
