@@ -1,5 +1,5 @@
 /* ----------------------------------------------------------------------- *
- *   
+ *
  *   Copyright 1996-2016 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
@@ -14,7 +14,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- *     
+ *
  *     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
  *     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
  *     INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
@@ -73,12 +73,14 @@
 #include <time.h>
 #include <stdarg.h>             /* Note: we need the ANSI version of stdarg.h */
 #include <ctype.h>
-#include <inttypes.h>
 
 #include "nasm.h"
 #include "nasmlib.h"
-#include "output/outform.h"
-#include "output/outlib.h"
+#include "error.h"
+#include "ver.h"
+
+#include "outform.h"
+#include "outlib.h"
 
 #ifdef OF_IEEE
 
@@ -184,8 +186,8 @@ struct ieeeFixupp {
 static int32_t ieee_entry_seg, ieee_entry_ofs;
 static int checksum;
 
-extern struct ofmt of_ieee;
-static struct dfmt ladsoft_debug_form;
+extern const struct ofmt of_ieee;
+static const struct dfmt ladsoft_debug_form;
 
 static void ieee_data_new(struct ieeeSection *);
 static void ieee_write_fixup(int32_t, int32_t, struct ieeeSection *,
@@ -222,14 +224,6 @@ static void ieee_init(void)
     checksum = 0;
 }
 
-static int ieee_set_info(enum geninfo type, char **val)
-{
-    (void)type;
-    (void)val;
-
-    return 0;
-}
-
 /*
  * Rundown
  */
@@ -786,7 +780,7 @@ static int32_t ieee_segment(char *name, int pass, int *bits)
                     nasm_error(ERR_NONFATAL, "segment alignment should be"
                           " numeric");
                 }
-                switch ((int)seg->align) {
+                switch (seg->align) {
                 case 1:        /* BYTE */
                 case 2:        /* WORD */
                 case 4:        /* DWORD */
@@ -830,7 +824,8 @@ static int32_t ieee_segment(char *name, int pass, int *bits)
 /*
  * directives supported
  */
-static int ieee_directive(enum directives directive, char *value, int pass)
+static enum directive_result
+ieee_directive(enum directive directive, char *value, int pass)
 {
 
     (void)value;
@@ -839,10 +834,10 @@ static int ieee_directive(enum directives directive, char *value, int pass)
     switch (directive) {
     case D_UPPERCASE:
         ieee_uppercase = true;
-        return 1;
-	
+        return DIRR_OK;
+
     default:
-	return 0;
+	return DIRR_UNKNOWN;
     }
 }
 
@@ -902,8 +897,7 @@ static void ieee_filename(char *inname, char *outname)
 
 static void ieee_write_file(void)
 {
-    struct tm *thetime;
-    time_t reltime;
+    const struct tm * const thetime = &official_compile_time.local;
     struct FileName *fn;
     struct ieeeSection *seg;
     struct ieeePublic *pub, *loc;
@@ -932,8 +926,6 @@ static void ieee_write_file(void)
     /*
      * date and time
      */
-    time(&reltime);
-    thetime = localtime(&reltime);
     ieee_putascii("DT%04d%02d%02d%02d%02d%02d.\n",
                   1900 + thetime->tm_year, thetime->tm_mon + 1,
                   thetime->tm_mday, thetime->tm_hour, thetime->tm_min,
@@ -1495,7 +1487,7 @@ static void dbgls_output(int output_type, void *param)
     (void)output_type;
     (void)param;
 }
-static struct dfmt ladsoft_debug_form = {
+static const struct dfmt ladsoft_debug_form = {
     "LADsoft Debug Records",
     "ladsoft",
     dbgls_init,
@@ -1505,13 +1497,14 @@ static struct dfmt ladsoft_debug_form = {
     dbgls_typevalue,
     dbgls_output,
     dbgls_cleanup,
+    NULL                        /* pragma list */
 };
-static struct dfmt *ladsoft_debug_arr[3] = {
+static const struct dfmt * const ladsoft_debug_arr[3] = {
     &ladsoft_debug_form,
     &null_debug_form,
     NULL
 };
-struct ofmt of_ieee = {
+const struct ofmt of_ieee = {
     "IEEE-695 (LADsoft variant) object file format",
     "ieee",
     OFMT_TEXT,
@@ -1520,7 +1513,7 @@ struct ofmt of_ieee = {
     &ladsoft_debug_form,
     NULL,
     ieee_init,
-    ieee_set_info,
+    nasm_do_legacy_output,
     ieee_out,
     ieee_deflabel,
     ieee_segment,
@@ -1528,7 +1521,8 @@ struct ofmt of_ieee = {
     ieee_segbase,
     ieee_directive,
     ieee_filename,
-    ieee_cleanup
+    ieee_cleanup,
+    NULL                        /* pragma list */
 };
 
 #endif                          /* OF_IEEE */
